@@ -30,6 +30,12 @@ typedef struct _md_finder {
     PyObject *identity;  // borrowed ref
 } md_finder_t;
 
+typedef enum _UpdateOp {
+    Extend,
+    Update,
+    Merge,
+} UpdateOp;
+
 /*
 The multidict's implementation is close to Python's dict except for multiple
 keys.
@@ -80,12 +86,12 @@ GROWTH_RATE(MultiDictObject *md)
     return md->used * 3;
 }
 
+#ifndef NDEBUG
 static inline int
 _md_check_consistency(MultiDictObject *md, bool update);
 static inline int
 _md_dump(MultiDictObject *md);
 
-#ifndef NDEBUG
 #define ASSERT_CONSISTENT(md, update) assert(_md_check_consistency(md, update))
 #else
 #define ASSERT_CONSISTENT(md, update) assert(1)
@@ -145,10 +151,10 @@ _ci_key_to_identity(mod_state *state, PyObject *key)
         }
         return ret;
     }
-fail:
     PyErr_SetString(PyExc_TypeError,
                     "CIMultiDict keys should be either str "
                     "or subclasses of str");
+fail:
     return NULL;
 }
 
@@ -209,7 +215,8 @@ _md_resize(MultiDictObject *md, uint8_t log2_newsize, bool update)
     } else {
         entry_t *new_ep = newentries;
         entry_t *old_ep = oldentries;
-        for (Py_ssize_t i = 0; i < oldkeys->nentries; ++i, ++old_ep) {
+        Py_ssize_t oldnumentries = oldkeys->nentries;
+        for (Py_ssize_t i = 0; i < oldnumentries; ++i, ++old_ep) {
             if (old_ep->identity != NULL) {
                 *new_ep++ = *old_ep;
             }
@@ -232,16 +239,54 @@ _md_resize(MultiDictObject *md, uint8_t log2_newsize, bool update)
     return 0;
 }
 
+static inline int
+_md_shrink(MultiDictObject *md, bool update)
+{
+    htkeys_t *keys = md->keys;
+    Py_ssize_t nentries = keys->nentries;
+    entry_t *entries = htkeys_entries(keys);
+    entry_t *new_ep = entries;
+    entry_t *old_ep = entries;
+    Py_ssize_t newnentries = nentries;
+    for (Py_ssize_t i = 0; i < nentries; ++i, ++old_ep) {
+        if (old_ep->identity != NULL) {
+            if (new_ep != old_ep) {
+                *new_ep = *old_ep;
+            }
+            new_ep++;
+        } else {
+            newnentries -= 1;
+        }
+    }
+    keys->nentries = newnentries;
+    keys->usable += nentries - newnentries;
+    memset(&keys->indices[0], 0xff, ((size_t)1 << keys->log2_index_bytes));
+    memset(new_ep, 0, sizeof(entry_t) * (size_t)(nentries - newnentries));
+    if (htkeys_build_indices(keys, entries, newnentries, update) < 0) {
+        return -1;
+    }
+    ASSERT_CONSISTENT(md, update);
+    return 0;
+}
+
 static inline int
 _md_resize_for_insert(MultiDictObject *md)
 {
-    return _md_resize(md, calculate_log2_keysize(GROWTH_RATE(md)), false);
+    if (md->used < md->keys->nentries) {
+        return _md_shrink(md, false);
+    } else {
+        return _md_resize(md, calculate_log2_keysize(GROWTH_RATE(md)), false);
+    }
 }
 
 static inline int
 _md_resize_for_update(MultiDictObject *md)
 {
-    return _md_resize(md, calculate_log2_keysize(GROWTH_RATE(md)), true);
+    if (md->used < md->keys->nentries) {
+        return _md_shrink(md, true);
+    } else {
+        return _md_resize(md, calculate_log2_keysize(GROWTH_RATE(md)), true);
+    }
 }
 
 static inline int
@@ -830,6 +875,8 @@ fail:
 static inline int
 md_get_all(MultiDictObject *md, PyObject *key, PyObject **ret)
 {
+    int tmp;
+    PyObject *value = NULL;
     *ret = NULL;
 
     md_finder_t finder = {0};
@@ -844,9 +891,6 @@ md_get_all(MultiDictObject *md, PyObject *key, PyObject **ret)
         goto fail;
     }
 
-    int tmp;
-    PyObject *value = NULL;
-
     while ((tmp = md_find_next(&finder, NULL, &value)) > 0) {
         if (*ret == NULL) {
             *ret = PyList_New(1);
@@ -866,7 +910,10 @@ md_get_all(MultiDictObject *md, PyObject *key, PyObject **ret)
         goto fail;
     }
 
-    md_finder_cleanup(&finder);
+    if (*ret != NULL) {
+        // there is no need to restore hashes if none was marked
+        md_finder_cleanup(&finder);
+    }
     Py_DECREF(identity);
     return *ret != NULL;
 fail:
@@ -1166,7 +1213,7 @@ _md_update(MultiDictObject *md, Py_hash_t hash, PyObject *identity,
     bool found = false;
 
     for (; iter.index != DKIX_EMPTY; htkeysiter_next(&iter)) {
-        if (iter.index == DKIX_DUMMY) {
+        if (iter.index < 0) {
             continue;
         }
         entry_t *entry = entries + iter.index;
@@ -1210,6 +1257,38 @@ fail:
 }
 
 static inline int
+_md_merge(MultiDictObject *md, Py_hash_t hash, PyObject *identity,
+          PyObject *key, PyObject *value)
+{
+    htkeysiter_t iter;
+    htkeysiter_init(&iter, md->keys, hash);
+    entry_t *entries = htkeys_entries(md->keys);
+
+    for (; iter.index != DKIX_EMPTY; htkeysiter_next(&iter)) {
+        if (iter.index < 0) {
+            continue;
+        }
+        entry_t *entry = entries + iter.index;
+        if (hash != entry->hash) {
+            continue;
+        }
+        int tmp = _str_cmp(identity, entry->identity);
+        if (tmp > 0) {
+            return 0;
+        } else if (tmp < 0) {
+            goto fail;
+        }
+    }
+
+    if (_md_add_for_upd(md, hash, identity, key, value) < 0) {
+        goto fail;
+    }
+    return 0;
+fail:
+    return -1;
+}
+
+static inline void
 md_post_update(MultiDictObject *md)
 {
     htkeys_t *keys = md->keys;
@@ -1228,18 +1307,15 @@ md_post_update(MultiDictObject *md)
             }
             if (entry->hash == -1) {
                 entry->hash = _unicode_hash(entry->identity);
-                if (entry->hash == -1) {
-                    // hash of string always exists but still
-                    return -1;
-                }
             }
+            assert(entry->hash != -1);
         }
     }
-    return 0;
+    ASSERT_CONSISTENT(md, false);
 }
 
 static inline int
-md_update_from_ht(MultiDictObject *md, MultiDictObject *other, bool update)
+md_update_from_ht(MultiDictObject *md, MultiDictObject *other, UpdateOp op)
 {
     Py_ssize_t pos;
     Py_hash_t hash;
@@ -1277,14 +1353,23 @@ md_update_from_ht(MultiDictObject *md, MultiDictObject *other, bool update)
             hash = entry->hash;
             key = entry->key;
         }
-        if (update) {
-            if (_md_update(md, hash, identity, key, entry->value) < 0) {
-                goto fail;
-            }
-        } else {
-            if (_md_add_with_hash(md, hash, identity, key, entry->value) < 0) {
-                goto fail;
-            }
+        switch (op) {
+            case Update:
+                if (_md_update(md, hash, identity, key, entry->value) < 0) {
+                    goto fail;
+                }
+                break;
+            case Extend:
+                if (_md_add_with_hash(md, hash, identity, key, entry->value) <
+                    0) {
+                    goto fail;
+                }
+                break;
+            case Merge:
+                if (_md_merge(md, hash, identity, key, entry->value) < 0) {
+                    goto fail;
+                }
+                break;
         }
         if (recalc_identity) {
             Py_CLEAR(identity);
@@ -1301,7 +1386,7 @@ fail:
 }
 
 static inline int
-md_update_from_dict(MultiDictObject *md, PyObject *kwds, bool update)
+md_update_from_dict(MultiDictObject *md, PyObject *kwds, UpdateOp op)
 {
     Py_ssize_t pos = 0;
     PyObject *identity = NULL;
@@ -1321,22 +1406,36 @@ md_update_from_dict(MultiDictObject *md, PyObject *kwds, bool update)
         if (hash == -1) {
             goto fail;
         }
-        if (update) {
-            if (_md_update(md, hash, identity, key, value) < 0) {
-                goto fail;
+        switch (op) {
+            case Update: {
+                if (_md_update(md, hash, identity, key, value) < 0) {
+                    goto fail;
+                }
+                Py_CLEAR(identity);
+                Py_CLEAR(key);
+                break;
             }
-            Py_CLEAR(identity);
-            Py_CLEAR(key);
-        } else {
-            int tmp = _md_add_with_hash_steal_refs(
-                md, hash, identity, key, Py_NewRef(value));
-            if (tmp < 0) {
-                Py_DECREF(value);
-                goto fail;
+            case Extend: {
+                int tmp = _md_add_with_hash_steal_refs(
+                    md, hash, identity, key, Py_NewRef(value));
+                if (tmp < 0) {
+                    Py_DECREF(value);
+                    goto fail;
+                }
+
+                identity = NULL;
+                key = NULL;
+                value = NULL;
+                break;
+            }
+            case Merge: {
+                if (_md_merge(md, hash, identity, key, value) < 0) {
+                    goto fail;
+                }
+                Py_CLEAR(identity);
+                Py_CLEAR(key);
+                break;
             }
-            identity = NULL;
-            key = NULL;
-            value = NULL;
         }
     }
     return 0;
@@ -1426,7 +1525,7 @@ fail:
 }
 
 static inline int
-md_update_from_seq(MultiDictObject *md, PyObject *seq, bool update)
+md_update_from_seq(MultiDictObject *md, PyObject *seq, UpdateOp op)
 {
     PyObject *it = NULL;
     PyObject *item = NULL;  // seq[i]
@@ -1506,21 +1605,32 @@ md_update_from_seq(MultiDictObject *md, PyObject *seq, bool update)
             goto fail;
         }
 
-        if (update) {
-            if (_md_update(md, hash, identity, key, value) < 0) {
-                goto fail;
-            }
-            Py_CLEAR(identity);
-            Py_CLEAR(key);
-            Py_CLEAR(value);
-        } else {
-            if (_md_add_with_hash_steal_refs(md, hash, identity, key, value) <
-                0) {
-                goto fail;
-            }
-            identity = NULL;
-            key = NULL;
-            value = NULL;
+        switch (op) {
+            case Update:
+                if (_md_update(md, hash, identity, key, value) < 0) {
+                    goto fail;
+                }
+                Py_CLEAR(identity);
+                Py_CLEAR(key);
+                Py_CLEAR(value);
+                break;
+            case Extend:
+                if (_md_add_with_hash_steal_refs(
+                        md, hash, identity, key, value) < 0) {
+                    goto fail;
+                }
+                identity = NULL;
+                key = NULL;
+                value = NULL;
+                break;
+            case Merge:
+                if (_md_merge(md, hash, identity, key, value) < 0) {
+                    goto fail;
+                }
+                Py_CLEAR(identity);
+                Py_CLEAR(key);
+                Py_CLEAR(value);
+                break;
         }
         Py_CLEAR(item);
     }
@@ -1767,7 +1877,7 @@ md_traverse(MultiDictObject *md, visitproc visit, void *arg)
 static inline int
 md_clear(MultiDictObject *md)
 {
-    if (md->used == 0) {
+    if (md->keys == NULL || md->keys == &empty_htkeys) {
         return 0;
     }
     md->version = NEXT_VERSION(md->state);
@@ -1791,6 +1901,8 @@ md_clear(MultiDictObject *md)
     return 0;
 }
 
+#ifndef NDEBUG
+
 static inline int
 _md_check_consistency(MultiDictObject *md, bool update)
 {
@@ -1852,7 +1964,7 @@ static inline int
 _md_dump(MultiDictObject *md)
 {
     htkeys_t *keys = md->keys;
-    printf("Dump %p [%ld from %ld usable %ld nentries %ld]\n",
+    printf("Dump %p [%zd from %zd usable %zd nentries %zd]\n",
            (void *)md,
            md->used,
            htkeys_nslots(keys),
@@ -1860,7 +1972,7 @@ _md_dump(MultiDictObject *md)
            keys->nentries);
     for (Py_ssize_t i = 0; i < htkeys_nslots(keys); i++) {
         Py_ssize_t ix = htkeys_get_index(keys, i);
-        printf("  %ld -> %ld\n", i, ix);
+        printf("  %zd -> %zd\n", i, ix);
     }
     printf("  --------\n");
     entry_t *entries = htkeys_entries(keys);
@@ -1869,9 +1981,9 @@ _md_dump(MultiDictObject *md)
         PyObject *identity = entry->identity;
 
         if (identity == NULL) {
-            printf("  %ld [deleted]\n", i);
+            printf("  %zd [deleted]\n", i);
         } else {
-            printf("  %ld h=%20ld, i=\'", i, entry->hash);
+            printf("  %zd h=%20zd, i=\'", i, entry->hash);
             PyObject_Print(entry->identity, stdout, Py_PRINT_RAW);
             printf("\', k=\'");
             PyObject_Print(entry->key, stdout, Py_PRINT_RAW);
@@ -1883,6 +1995,7 @@ _md_dump(MultiDictObject *md)
     printf("\n");
     return 1;
 }
+#endif  // NDEBUG
 
 #ifdef __cplusplus
 }
